/* dem2tga.c (c) 1997 Jon Larimer
 *
 *  This is a program to convert USGS Digital Elevation Model (DEM) data
 *  into a .TGA file, viewable by many graphics applications.  This program
 *  is public domain, do whatever you want with it, except claim it as your
 *  own or sell it. If you find a way to make money from my work, I want a
 *  piece of the action.
 *
 *  This should compile with any ANSI standard compiler, including Unix
 *  and DOS versions of gcc and many other C compilers.  If you have any
 *  questions or comments email jonl@alltel.net.
 *
 *  Try: dem2tga santa_ana-w santa_ana.tga
 */


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/errno.h>

int writetgaheader(FILE *fptr, int r, int c) {
  int i;
 
  fputc(0, fptr);
  fputc(1, fptr);
  fputc(1, fptr);
  fputc(0, fptr);
  fputc(0, fptr);
  fputc(0, fptr);
  fputc(1, fptr);
  fputc(24, fptr);
  fputc(0, fptr);
  fputc(0, fptr);
  fputc(0, fptr);
  fputc(0, fptr);
  fputc((char)(c & 0x00ff), fptr);
  fputc((char)((c & 0xff00) >> 8), fptr);
  fputc((char)(r & 0x00ff), fptr);
  fputc((char)((r & 0xff00) >> 8), fptr);
  fputc(8, fptr);
  fputc(0, fptr);
  for(i=0; i<=255; i++) {
    fputc(i, fptr);
    fputc(i, fptr);
    fputc(i, fptr);
  }
  fflush(fptr);
  return 0;
}

int nextint(FILE * fptr) {
  char ch;
  int i;
  char in[64];

  while(isspace(ch = fgetc(fptr))) { }
  i=0;
  in[0] = ch;
  while(!isspace(ch = fgetc(fptr))) { i++; in[i] = ch; }
  in[i+1] = '\0';
  return(atoi(in));
} 
          
double nextfloat(FILE * fptr) {
  char ch;
  int i;
  char in[64];
  
  while(isspace(ch = fgetc(fptr))) { }
  i=0;
  in[0] = ch;
  while(!isspace(ch = fgetc(fptr))) { 
    i++; 
    if(ch == 'D') { ch = 'E'; }
    in[i] = ch; 
  }
  in[i+1] = '\0';
  atof(in);
}

int main(int argc, char **argv) {
  FILE * demfile;
  FILE * outfile;
  int i, j, l;
  float k;
  int maxelev, minelev;
  int rows, columns;
  int elevrange;
  float colormap_scale;
  int r, c;
  int cur_r, cur_c;
  char ch;
  char name[144];
  
  fprintf(stdout, "dem2tga v1.0 (c) 1997 Jon Larimer\n");
  fprintf(stdout, "This program is public domain.\n\n");

  if(argc < 2) {
    fprintf(stdout, "usage: dem2tga input_file output_file\n");
    fprintf(stdout, "  input_file is the DEM you want to convert\n");
    fprintf(stdout, "  output_file is the name you want to save the image under.\n");
    fprintf(stdout, "    You should supply a .tga at the end of the file.\n");
    exit(1);
  }

  if((demfile = fopen(argv[1], "r")) == NULL) {
    fprintf(stderr, "%s: fopen: %s", argv[0], strerror(errno));
    exit(1);
  }
  
  /****** DEM TYPE A RECORDS ******/
  /* get the name field (144 characters) */
  for(i=0; i < 144; i++) { name[i] = fgetc(demfile); }
  name[i+1] = '\0';
  /* clean off the whitespace at the end */
  for(i=strlen(name)-2; i > 0; i--) {
    if(!isspace(name[i])) { i=0; } else { name[i] = '\0'; }
   
  }
  fprintf(stdout, "Quad name field: %s\n", name);
  /* don't need the next 19 items for anything */
  for(i=0; i < 19; i++) { (void)nextint(demfile); }
  fprintf(stdout, "Units code (ground planametric coordinates): %i\n", nextint(demfile));
  fprintf(stdout, "Units code (elevation coordinates): %i\n", nextint(demfile));
  (void)nextint(demfile);
  printf("Ground coordinates of 4 corners of DEM: (in arc-seconds) \n");
  for(i=1; i<=4; i++)
    fprintf(stdout, "  %.4f  %.4f\n", nextfloat(demfile), nextfloat(demfile));
  minelev = (int)nextfloat(demfile);
  maxelev = (int)nextfloat(demfile);
  fprintf(stdout, "\nMinimum elevation: %i\n", minelev);
  fprintf(stdout, "Maximum elevation: %i\n", maxelev);
  elevrange = maxelev-minelev;
  fprintf(stdout, "Elevation range: %i\n", elevrange);
  /* this is the value to multiply elevations by to get a number between 0
     and 255 to determine the color of the pixel */
  colormap_scale = (float)255 / (float)elevrange;
  fprintf(stdout, "Colormap scaling value: %.5f\n", colormap_scale);
  /* don't need the next 3 items */
  for(i=1; i<3; i++) { (void)nextint(demfile); }
  cur_r = nextint(demfile);
  columns = nextint(demfile);
  /* DEM TYPE B RECORDS */
  cur_c = 1;
  fprintf(stdout, "Writing image...");
  fflush(stdout);
  outfile = fopen(argv[2], "wb+");
  l = 1;
  /* we need to see how many rows of data before we write the header */
  cur_r = nextint(demfile);
  cur_c = nextint(demfile);
  rows = nextint(demfile);
  writetgaheader(outfile, rows, columns);
  /* now ready to write the image data */
  while(cur_c < columns) {
/*    if(l==1) {
      l=0;
    } else {
      cur_r = nextint(demfile);
      cur_c = nextint(demfile);
      rows = nextint(demfile);
    }
    */
    (void)nextint(demfile); (void)nextint(demfile);
    (void)nextint(demfile); (void)nextint(demfile);
    (void)nextint(demfile); (void)nextint(demfile);
    for(i=1; i<=rows; i++) { 
      j = nextint(demfile) - minelev; 
      k = (float)j * colormap_scale;
      fputc((int)k, outfile);
    }
    cur_r = nextint(demfile);
    cur_c = nextint(demfile);
    rows = nextint(demfile);
  }
  fprintf(stdout, "done.\n");
  fclose(outfile);
  fclose(demfile);
  return 0;
}
  
  
